__all__ = ['pc_query', 'pc_drop', 'pc_temp']

from ._imports import *
from ._config import config
#from ._cyberark import get_credential_both_method

class Pc:

    conn = None

    @classmethod
    def get_conn(cls):
        if cls.conn is None:
            # user, password = get_credential_both_method('policycenter')
            url = "mssql+pymssql://{user}:{password}@{host}/{database}".format(
                user = config['policycenter']['user'], 
                password = config['policycenter']['password'],  
                host =  config['policycenter']['host'],  
                database = config['policycenter']['database'],  
            )
            cls.conn = sqlalchemy.create_engine(url)
        return cls.conn

def pc_conn():
    return Pc.get_conn()

def pc_query(q):
    return pd.read_sql(q, pc_conn())

def pc_drop(table_name):
    try:
        pc_conn().execute(f'drop table {table_name}')
    except:
        pass

def pc_temp(df, table_name):
    pc_drop(table_name)
    df.to_sql(name=table_name, con=pc_conn(), index=False)
    q = f'select * from {table_name}'
    return pc_query(q)

@atexit.register
def pc_close():
    if Pc.conn:
        Pc.conn.dispose()



__all__ = [
    'athena_query',
]

from ._imports import *
from ._config import config
from ._amazons3 import *
#from ._cyberark import *

class AmazonAthena:
    clients = {}

    @classmethod
    def get_client(cls, env):
        if env not in cls.clients:
            if config.has_option(f'aws_{env}','aws_session_token'):
                cls.clients[env] = boto3.client(
                    service_name            = 'athena',
                    region_name             = config[f'aws_{env}']['region'], 
                    aws_access_key_id       = config[f'aws_{env}']['aws_access_key_id'],
                    aws_secret_access_key   = config[f'aws_{env}']['aws_secret_access_key'],
                    aws_session_token       = config[f'aws_{env}']['aws_session_token']
                )
            else:
                #aws_access_key_id, aws_secret_access_key  = get_credential_both_method(f'aws_{env}')
                cls.clients[env] = boto3.client(
                    service_name            = 'athena',
                    region_name             = config[f'aws_{env}']['region'], 
                    aws_access_key_id       = config[f'aws_{env}']['aws_access_key_id'],
                    aws_secret_access_key   = config[f'aws_{env}']['aws_secret_access_key'],
                    # aws_access_key_id  = aws_access_key_id,
                    # aws_secret_access_key  = aws_secret_access_key 
                )
        return cls.clients[env]

def client(env):
    return AmazonAthena.get_client(env)

@atexit.register
def athena_close():
    for client in AmazonAthena.clients.values():
        client.close()

def athena_query(query, env='dl', path=None, cleanup=False, time_sleep=1, max_execution=100):
    path = config[f'aws_{env}']['aws_folder'] if path is None else path
    params = {
        'env':       env,
        'region':    config[f'aws_{env}']['region'],
        'bucket':    config[f'aws_{env}']['bucket'],
        'database':  'amica_eds_digital',
        'workgroup': 'primary',
        'path':      path,
        'query':     query,
    }

    athena_client = client(env)
    execution = athena_client.start_query_execution(
        QueryString=query,
        QueryExecutionContext={
            'Database': params['database']
        },
        ResultConfiguration={
            'OutputLocation': 's3://' + params['bucket'] + '/' + params['path']
        }
    )
    execution_id = execution['QueryExecutionId']

    state = 'RUNNING'
    while max_execution > 0 and state in ['RUNNING', 'QUEUED']:
        max_execution = max_execution - 1
        response = athena_client.get_query_execution(QueryExecutionId=execution_id)

        if 'QueryExecution' in response and \
                'Status' in response['QueryExecution'] and \
                'State' in response['QueryExecution']['Status']:
            state = response['QueryExecution']['Status']['State']

            if state == 'FAILED':
                raise Exception('Athena Query Failed')
            elif state == 'SUCCEEDED':
                s3_path = response['QueryExecution']['ResultConfiguration']['OutputLocation']
                filename = re.findall('.*\/(.*)', s3_path)[0]

                s3_file_path = os.path.join(path, filename)
                df = s3_download_and_read_csv(s3_file_path, s3_bucket=params['bucket'], env=env)
                if cleanup:
                    s3_delete_file(s3_file_path, s3_bucket=params['bucket'], env=env)
                return df
        time.sleep(time_sleep)



from ._config import *
from ._amazons3 import *
from ._amazonathena import *
from ._claimcenter import *
from ._datetime import *
from ._genesys import *
from ._policycenter import *
from ._salesforce import *
from ._teradata import *




__all__ = ['cc_query', 'cc_query_table']

from ._imports import *
from ._config import config
#from ._cyberark import get_credential_both_method

class Cc:

    conn = None

    @classmethod
    def query(cls, q):
        if cls.conn is None:            
            url = "postgresql+psycopg2://{user}:{password}@{host}/{database}".format(
                user = config['claimcenter']['user'], 
                password =  config['claimcenter']['password'], 
                host = config['claimcenter']['host'],  
                database = config['claimcenter']['database']  
            )
            cls.conn = sqlalchemy.create_engine(url)
        return pd.read_sql(q, cls.conn)
  
def cc_query(q):
    return Cc.query(q)

def cc_query_table(database='cc_merge', table=None, column=None, limit=None):
    q = f'''
        select table_schema, table_name, column_name
        from information_schema.columns
        where table_schema like \'{database}\'
            {f"and table_name like '{table}'" if table else ""}
            {f"and column_name like '{column}'" if column else ""}
        {f"limit {limit}" if limit else ""}
    '''
    return cc_query(q)

@atexit.register
def cc_close():
    if Cc.conn:
        # Cc.conn.close()
        Cc.conn.dispose()


__all__ = ['DtFind']

from ._imports import *

class DtFind:

    @staticmethod
    def previous_day(dt_, dt_format=None):
        d = dt_ - dt.timedelta(1)
        if dt_format:
            d = d.strftime(dt_format)
        return d

    @staticmethod
    def first_day_of_month(dt_, dt_format=None):
        d = dt_.replace(day=1)
        if dt_format:
            d = d.strftime(dt_format)
        return d
    
    @classmethod
    def first_day_last_month(cls, dt_, dt_format=None):
        d = cls.first_day_of_month(cls.first_day_of_month(dt_) - dt.timedelta(days=1))
        if dt_format:
            d = d.strftime(dt_format)
        return d
    
    @classmethod
    def first_day_next_month(cls, dt_, dt_format=None):
        d = cls.first_day_of_month(dt_.replace(day=28) + dt.timedelta(days=4))
        if dt_format:
            d = d.strftime(dt_format)
        return d
    
    @classmethod
    def last_day_of_month(cls, dt_, dt_format=None):
        d = cls.first_day_next_month(dt_) - dt.timedelta(days=1)
        if dt_format:
            d = d.strftime(dt_format)
        return d



__all__ = ['query_creator', 'td_query', 'td_volatile', 'td_drop_table', 'td_query_table']

from ._imports import *
from ._config import config
#from ._cyberark import get_credential_both_method

class Td:

    engine = None

    @classmethod
    def get_engine(cls):
        if cls.engine is None:            
            # udaExec = teradata.UdaExec (appName="edsds", version="1.0",logConsole=False)
            # cls.engine = udaExec.connect(method="odbc", 
            #     system=config['teradata']['system'],
            #     username= config['teradata']['user'],  
            #     password= config['teradata']['password'], 
            #     authentication="LDAP", 
            #     driver =  config['teradata']['driver']
            #     )
            user = config['teradata']['user']
            password= config['teradata']['password']
            host = config['teradata']['system']
            cls.engine = sqlalchemy.create_engine(
                 f'teradatasql://{user}:{password}@{host}/?logmech=ldap&encryptdata=true',echo=False)
        return cls.engine
    
@atexit.register
def td_close():
    if Td.engine:
        Td.engine.dispose()

def query_creator(table_columns='*', table_name='atuprv.dw_agreement', table_ops='sample 3'):
     q = f'select {table_columns} from {table_name} {table_ops}'
     return q
    
def td_query(q):
    return pd.read_sql(q, Td.get_engine())

def td_volatile(q, table_name):
    q = f'''
            create volatile multiset table {table_name} as (
                {q}
            ) with data
            no primary index
            on commit preserve rows;
            '''
    df = Td.get_engine().execute(q)
    return df

def td_drop_table(table_name):
    q = f'drop table {table_name};'
    try:
        Td.get_engine().execute(q)
    except:
        pass
    return

def td_query_table(database='atuprv', table=None, column=None, sample=None):
    q = f'''
        select databasename, tablename, columnname
        from dbc.columnsv
        where databasename like \'{database}\'
            {f"and tablename like '{table}'" if table else ""}
            {f"and columnname like '{column}'" if column else ""}
        {f"sample {sample}" if sample else ""}
    '''
    return td_query(q)




from setuptools import setup, find_packages

setup(name='eds_ds_tools',
      version='1.0',
      description='tools for data science',
      #packages=['eds_ds_tools'],
      author_email='wenleicao@gmail.com',
      zip_safe=False,      
      packages=find_packages(where="."),
        package_dir={"": "."},
        package_data={
            'eds_ds_tools': ['*.zip'], # Include all .zip files in 'eds_ds_tools'
        },
        include_package_data=True
      
      )
